<?php

/**
 * Imgur extension for phpBB.
 * @author Alfredo Ramos <alfredo.ramos@yandex.com>
 * @copyright 2017 Alfredo Ramos
 * @license GPL-2.0-only
 */

namespace alfredoramos\imgur\controller;

use phpbb\auth\auth;
use phpbb\config\config;
use phpbb\template\template;
use phpbb\request\request;
use phpbb\controller\helper;
use phpbb\filesystem\filesystem;
use phpbb\language\language;
use phpbb\user;
use phpbb\log\log;
use phpbb\exception\runtime_exception;
use phpbb\exception\http_exception;
use phpbb\request\request_interface;
use phpbb\json_response;
use Imgur\Client as ImgurClient;
use Imgur\Exception\ErrorException as ImgurErrorException;
use Symfony\Component\HttpFoundation\JsonResponse;

class imgur
{
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\filesystem\filesystem */
	protected $filesystem;

	/** @var \phpbb\language\language */
	protected $language;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\log\log */
	protected $log;

	/** @var \Imgur\Client */
	protected $imgur;

	/**
	 * Controller constructor.
	 *
	 * @param \phpbb\auth\auth				$auth
	 * @param \phpbb\config\config			$config
	 * @param \phpbb\template\template		$template
	 * @param \phpbb\request\request		$request
	 * @param \phpbb\controller\helper		$helper
	 * @param \phpbb\filesystem\filesystem	$filesystem
	 * @param \phpbb\language\language		$language
	 * @param \phpbb\user					$user
	 * @param \phpbb\log\log				$log
	 * @param \Imgur\Client					$imgur
	 *
	 * @return void
	 */
	public function __construct(auth $auth, config $config, template $template, request $request, helper $helper, filesystem $filesystem, language $language, user $user, log $log, ImgurClient $imgur)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->template = $template;
		$this->request = $request;
		$this->helper = $helper;
		$this->filesystem = $filesystem;
		$this->language = $language;
		$this->user = $user;
		$this->log = $log;
		$this->imgur = $imgur;

		// Add controller translations
		$this->language->add_lang(['controller', 'acp/info_acp_settings'], 'alfredoramos/imgur');

		// Mandatory API data
		if (empty($this->config['imgur_client_id']) || empty($this->config['imgur_client_secret']))
		{
			throw new runtime_exception('EXCEPTION_IMGUR_NO_API_DATA');
		}

		// Setup Imgur API
		$this->imgur->setOption('client_id', $this->config['imgur_client_id']);
		$this->imgur->setOption('client_secret', $this->config['imgur_client_secret']);
	}

	/**
	 * Imgur authorization controller handler.
	 *
	 * @param string $hash
	 *
	 * @return \Symfony\Component\HttpFoundation\Response
	 */
	public function authorize($hash = '')
	{
		// This route can only be used by admins
		// Users do not need to know this page exist
		if ((int) $this->auth->acl_get('a_') !== 1)
		{
			throw new http_exception(404, 'PAGE_NOT_FOUND');
		}

		// Get Imgur token
		$token = $this->imgur_token();

		// Parse response fom Imgur API
		if (!$this->request->is_ajax())
		{
			$this->template->assign_vars([
				'IMGUR_IS_AUTHORIZED' => (!empty($token['access_token']) && !empty($token['refresh_token'])),
				'IMGUR_AUTHORIZE_URL' => vsprintf('%1$s/%2$s', [
					$this->helper->route('alfredoramos_imgur_authorize'),
					generate_link_hash('imgur_authorize')
				])
			]);

			return $this->helper->render('imgur_authorize.html', $this->language->lang('IMGUR_AUTHORIZATION'));
		}

		// Security hash
		$hash = trim($hash);

		// CSRF protection
		if (empty($hash) || !check_link_hash($hash, 'imgur_authorize'))
		{
			throw new http_exception(403, 'NO_AUTH_OPERATION');
		}

		// Construct new token
		$new_token = [
			'access_token'		=> '',
			'expires_in'		=> 0,
			'token_type'		=> '',
			'refresh_token'		=> '',
			'account_id'		=> 0,
			'account_username'	=> ''
		];

		// Generate new token
		foreach ($new_token as $key => $value)
		{
			// Cast values
			$value = (in_array($key, ['expires_in', 'account_id'])) ? (int) $value : trim($value);

			// Generate new token
			$new_token[$key] = $this->request->variable($key, $value);
		}

		// Set token
		$this->imgur->setAccessToken($new_token);

		// Update token
		$token = $this->imgur->getAccessToken();

		// Save new token
		foreach ($token as $key => $value)
		{
			$this->config->set(sprintf('imgur_%s', $key), $value, false);
		}

		// Admin log
		$this->log->add(
			'admin',
			$this->user->data['user_id'],
			$this->user->ip,
			'LOG_IMGUR_DATA',
			false,
			[$this->language->lang('IMGUR_AUTHORIZATION')]
		);

		// Return a JSON response
		$response = new json_response;

		return $response->send(['message' => 'success']);
	}

	/**
	 * Upload controller handler. AJAX calls only.
	 *
	 * @param string $hash
	 *
	 * @return string
	 */
	public function upload($hash = '')
	{
		// This route only responds to AJAX calls
		if (!$this->request->is_ajax())
		{
			throw new runtime_exception('EXCEPTION_IMGUR_AJAX_ONLY');
		}

		// Security hash
		$hash = trim($hash);

		// CSRF protection
		if (empty($hash) || !check_link_hash($hash, 'imgur_upload'))
		{
			throw new http_exception(403, 'NO_AUTH_OPERATION');
		}

		// Get Imgur token
		$token = $this->imgur_token();

		// Set token
		$this->imgur->setAccessToken($token);

		// Check if token expired
		if ($this->imgur->checkAccessTokenExpired())
		{
			$this->refresh_token();
		}

		// Not using $request->file() because I need an array of arrays
		$images = $this->request->variable(
			'imgur_image',
			['' => [0 => '']],
			true,
			request_interface::FILES
		);

		// Upload responses
		$data = [];

		// Upload images
		if (!empty($images['tmp_name']))
		{
			foreach ($images['name'] as $key => $value)
			{
				// Image file must exist and be readable
				if (!$this->filesystem->is_readable($images['tmp_name'][$key]))
				{
					continue;
				}

				// Upload image and save response, it will be used
				// later to show a JSON object
				try
				{
					$data[] = $this->imgur->api('image')->upload([
						'image' => base64_encode(file_get_contents($images['tmp_name'][$key])),
						'type'	=> 'base64',
						'album'	=> $this->config['imgur_album'],
						'name'	=> $value
					]);
				}
				catch (ImgurErrorException $ex)
				{
					// Update token as it expired unexpectedly
					$this->refresh_token();

					// Clear previous data
					$data = [];

					// Try again to upload the image, if succeed it will add
					// the BBCode of the image, otherwise an error message
					try
					{
						$data[] = $this->imgur->api('image')->upload([
							'image' => base64_encode(file_get_contents($images['tmp_name'][$key])),
							'type'	=> 'base64',
							'album'	=> $this->config['imgur_album'],
							'name'	=> $value
						]);
					}
					catch (ImgurErrorException $ex)
					{
						throw new runtime_exception('EXCEPTION_IMGUR_BAD_REQUEST', [$ex->getMessage()], $ex);
					}
				}
			}
		}

		// Image errors
		if (!empty($images['error']))
		{
			$errors = [];

			foreach ($images['error'] as $key => $value)
			{
				$value = (int) $value;

				switch ($value)
				{
					case UPLOAD_ERR_INI_SIZE:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_INI_SIZE');
					break;

					case UPLOAD_ERR_FORM_SIZE:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_FORM_SIZE');
					break;

					case UPLOAD_ERR_PARTIAL:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_PARTIAL');
					break;

					case UPLOAD_ERR_NO_FILE:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_NO_FILE');
					break;

					case UPLOAD_ERR_NO_TMP_DIR:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_NO_TMP_DIR');
					break;

					case UPLOAD_ERR_CANT_WRITE:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_CANT_WRITE');
					break;

					case UPLOAD_ERR_EXTENSION:
						$errors[]['message'] = $this->language->lang('UPLOAD_ERR_EXTENSION');
					break;

					default: // UPLOAD_ERR_OK
						continue;
					break;
				}
			}

			if (!empty($errors))
			{
				return new JsonResponse($errors, 500);
			}
		}

		// Return a JSON response
		$response = new json_response;

		return $response->send($data);
	}

	/**
	 * Get Imgur token stored in database.
	 *
	 * @return array
	 */
	private function imgur_token()
	{
		return [
			'access_token'		=> $this->config['imgur_access_token'],
			'expires_in'		=> (int) $this->config['imgur_expires_in'],
			'token_type'		=> $this->config['imgur_token_type'],
			'refresh_token'		=> $this->config['imgur_refresh_token'],
			'account_id'		=> (int) $this->config['imgur_accound_id'],
			'account_username'	=> $this->config['imgur_account_username'],
			'created_at'		=> (int) $this->config['imgur_created_at']
		];
	}

	/**
	 * Refresh token helper.
	 *
	 * @return void
	 */
	private function refresh_oken()
	{
		$this->imgur->refreshToken();

		// Generate new token
		$new_token = array_merge($this->imgur->getAccessToken(), [
			'created_at' => time()
		]);

		// Update the token in database
		foreach ($new_token as $key => $value)
		{
			// Save changes
			$this->config->set(sprintf('imgur_%s', $key), $value, false);
		}
	}
}
